import React, { useState, useEffect } from 'react'
import { withClient, Client } from 'cozy-client'
import { translate } from 'cozy-ui/react/I18n'

import IFluidConfig from 'services/IFluidConfig'

import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import StyledButton from 'components/CommonKit/Button/StyledButton'
import TrailingIcon from 'assets/icons/ico/trailing-icon.svg'
import { ConnectionService } from 'services/connectionService'
import { AccountService } from 'services/accountService'
import { Account, AuthLoginData, Trigger } from 'doctypes'

import iconGRDFLogo from 'assets/icons/visu/grdf-logo.svg'
import iconEGLLogo from 'assets/icons/visu/egl-logo.svg'
import StyledAuthButton from 'components/CommonKit/Button/StyledAuthButton'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'

interface KonnectorLoginFormProps {
  fluidConfig: IFluidConfig
  onSuccess: Function
  account: Account
  trigger: Trigger
  client: Client
  t: Function
}

const KonnectorLoginForm: React.FC<KonnectorLoginFormProps> = ({
  fluidConfig,
  onSuccess,
  account,
  trigger,
  client,
  t,
}: KonnectorLoginFormProps) => {
  const konnectorName: string = fluidConfig.konnectorConfig.name
  const konnectorType: string = fluidConfig.konnectorConfig.type
  const siteLink: string = fluidConfig.siteLink

  const [login, setLogin] = useState<string>('')
  const [password, setPassword] = useState<string>('')
  const [error, setError] = useState<string>('')
  const [loading, setLoading] = useState<boolean>(false)

  const getIcon = (fluidName: string) => {
    switch (fluidName) {
      case 'grdf':
        return iconGRDFLogo
      case 'egl':
        return iconEGLLogo
      default:
        return null
    }
  }

  const icon = getIcon(fluidConfig.name)

  const changeLogin = (value: string) => {
    setError('')
    setLogin(value)
  }

  const changePassword = (value: string) => {
    setError('')
    setPassword(value)
  }

  function revealPassword(idInput: string) {
    const input = document.getElementById(idInput)
    if (input) {
      if (input.getAttribute('type') === 'password') {
        input.setAttribute('type', 'text')
      } else {
        input.setAttribute('type', 'password')
      }
    }
  }

  const connect = async () => {
    const connectionService = new ConnectionService(
      client,
      fluidConfig.konnectorConfig.slug,
      login,
      password
    )
    const { account, trigger } = await connectionService.connectNewUser()
    if (!trigger) {
      setError(t('KONNECTORCONFIG.ERROR_ACCOUNT_CREATION'))
      setLoading(false)
      return null
    }
    onSuccess(account, trigger)
  }

  const update = async () => {
    const auth = {
      login: login,
      password: password,
    }
    account.auth = auth
    const updatedAccount = await AccountService.updateAccount(client, account)
    onSuccess(updatedAccount, trigger)
  }

  const handleSubmit = async (e: React.FormEvent<HTMLFormElement>) => {
    e.preventDefault()
    try {
      setError('')
      setLoading(true)
      if (!login || !password) {
        setError(t('KONNECTORCONFIG.ERROR_NO_LOGIN_PASSWORD'))
        setLoading(false)
        return null
      }
      if (!account) {
        await connect()
      } else {
        await update()
      }
    } catch (error) {
      setLoading(false)
    }
  }

  useEffect(() => {
    if (account) {
      const auth: AuthLoginData = account.auth
      if (auth.login) {
        setLogin(auth.login)
      }
      setError(t('KONNECTORCONFIG.ERROR_LOGIN_FAILED'))
    }
  }, [])

  return (
    <form
      className="form"
      onSubmit={(e: React.FormEvent<HTMLFormElement>) => handleSubmit(e)}
    >
      {t('KONNECTORCONFIG.LABEL_FILLIN')} {konnectorName}
      <div className="form-group">
        <input
          id={'idFieldLogin' + konnectorType}
          type="text"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-describedby="emailHelp"
          placeholder="Adresse e-mail"
          name="login"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changeLogin(e.target.value)
          }
          value={login}
        ></input>
      </div>
      <div className="form-group">
        <input
          id={'idFieldPassword' + konnectorType}
          type="password"
          className={
            error === ''
              ? 'form-control form-input'
              : 'form-control form-input --error'
          }
          aria-describedby="PasswordHelp"
          placeholder="Mot de passe"
          name="password"
          onChange={(e: React.ChangeEvent<HTMLInputElement>) =>
            changePassword(e.target.value)
          }
          value={password}
        />
        <span>
          <StyledIconButton
            icon={TrailingIcon}
            className="form-trailing-icon"
            size={22}
            onClick={() => revealPassword('idFieldPassword' + konnectorType)}
          />
        </span>
      </div>
      <div className="form-message">{error === '' ? null : error}</div>
      <div className="kloginauthform">
        <div className="kloginauthform-button">
          <StyledAuthButton
            className="form-button"
            type="submit"
            color="primary"
            disabled={loading}
          >
            <div className="authform-button-content">
              <div className="authform-button-content-icon">
                {icon && <StyledIcon icon={icon} size={48} />}
              </div>
              <div className="authform-button-text text-18-bold">
                <div>{t('auth.' + fluidConfig.name + '.connect.label1')}</div>
                <div>{t('auth.' + fluidConfig.name + '.connect.label2')}</div>
              </div>
            </div>
          </StyledAuthButton>
        </div>
        <div className="kloginauthform-text text-16-bold">
          <div className="text-16-bold">
            {t('auth.' + fluidConfig.name + '.no_account.title')}
          </div>
          <div className="text-16-normal">
            {t('auth.' + fluidConfig.name + '.no_account.text')}
          </div>
        </div>
        <StyledButton
          type="button"
          color="secondary"
          disabled={loading}
          onClick={() => window.open(siteLink, '_blank')}
        >
          {t('auth.' + fluidConfig.name + '.create_account')}
        </StyledButton>
      </div>
    </form>
  )
}

export default translate()(withClient(KonnectorLoginForm))
